@@ -11,15 +11,17 @@
 #include <poppler-page-renderer.h>
 #include <Rcpp.h>
 #include <cstring>
+#include <memory> //For std::unqiue_ptr in older gcc
 
 /* Note: 0.72.1 is my internal build of 0.72.0 + backported UTF-8 patches */
-#if defined(POPPLER_VERSION_MINOR) && (POPPLER_VERSION_MINOR >= 73 || POPPLER_VERSION_MAJOR > 0 || \
-  (POPPLER_VERSION_MINOR == 72 && POPPLER_VERSION_MICRO == 1))
+#if defined(POPPLER_VERSION_MINOR) && (POPPLER_VERSION_MINOR >= 73 || POPPLER_VERSION_MAJOR > 0)
 #define POPPLER_HAS_PAGE_TEXT_LIST
 #endif
 
-/* Note: Before poppler 0.73, UTF8 conversion was completely broken on MacOS */
-#if defined(POPPLER_HAS_PAGE_TEXT_LIST) || !defined(__APPLE__)
+/* Note: Before poppler 0.73, ustring to UTF8 conversion was unusable on non-linux
+ * due to an UTF-16 BE/LE bug. Therefore we better use to_latin1() on those systems,
+ * which works at least for ascii text */
+#if defined(POPPLER_HAS_PAGE_TEXT_LIST) || defined(__linux__)
 #define ustring_to_r ustring_to_utf8
 #else
 #define ustring_to_r ustring_to_latin1
@@ -28,24 +30,14 @@
 using namespace Rcpp;
 using namespace poppler;
 
-static char poppler_data[4000] = "";
-
 // [[Rcpp::export]]
-void set_poppler_data(std::string path){
-  strcpy(poppler_data, path.c_str());
-}
-
-// Call this after initiating document but before page
+bool set_poppler_data(std::string path){
 #ifdef BUNDLE_POPPLER_DATA
-#include <GlobalParams.h>
-void find_poppler_data(){
-  static bool initiated = false;
-  if (!initiated && strlen(poppler_data)){
-    globalParams = new GlobalParams(poppler_data);
-    initiated = true;
-  }
-}
+  return poppler::set_data_dir(path);
+#else
+  return false;
 #endif
+}
 
 String ustring_to_utf8(ustring x){
   byte_array buf = x.to_utf8();
@@ -62,7 +54,7 @@ String ustring_to_latin1(ustring x){
   return y;
 }
 
-List item_to_list(toc_item *item){
+static List item_to_list(toc_item *item){
   List out = List();
   std::vector <toc_item*> children = item->children();
   for(size_t i = 0; i < children.size(); i++){
@@ -74,7 +66,7 @@ List item_to_list(toc_item *item){
   );
 }
 
-std::string layout_string(document::page_layout_enum x) {
+static std::string layout_string(document::page_layout_enum x) {
   switch(x){
   case document::no_layout: return "no_layout";
   case document::single_page: return "single_page";
@@ -87,7 +79,7 @@ std::string layout_string(document::page_layout_enum x) {
   }
 }
 
-std::string font_string(font_info::type_enum x){
+static std::string font_string(font_info::type_enum x){
   switch(x) {
   case font_info::unknown: return "unknown";
   case font_info::type1: return "type1";
@@ -105,14 +97,11 @@ std::string font_string(font_info::type_enum x){
   }
 }
 
-document *read_raw_pdf(RawVector x, std::string opw, std::string upw){
+static document *read_raw_pdf(RawVector x, std::string opw, std::string upw, bool info_only = 0){
   document *doc = document::load_from_raw_data(	(const char*) x.begin(), x.length(), opw, upw);
-#ifdef BUNDLE_POPPLER_DATA
-  find_poppler_data();
-#endif
   if(!doc)
     throw std::runtime_error("PDF parsing failure.");
-  if(doc->is_locked())
+  if(!info_only && doc->is_locked())
     throw std::runtime_error("PDF file is locked. Invalid password?");
   return doc;
 }
@@ -122,15 +111,18 @@ List get_poppler_config(){
   return List::create(
     _["version"] = poppler::version_string(),
     _["can_render"] = page_renderer::can_render(),
+#ifdef POPPLER_HAS_PAGE_TEXT_LIST
+    _["has_pdf_data"] = true,
+#else
+    _["has_pdf_data"] = false,
+#endif
     _["supported_image_formats"] = image::supported_image_formats()
   );
 }
 
 // [[Rcpp::export]]
 List poppler_pdf_info (RawVector x, std::string opw, std::string upw) {
-  document *doc = document::load_from_raw_data(	(const char*) x.begin(), x.length(), opw, upw);
-  if(!doc)
-    throw std::runtime_error("PDF parsing failure.");
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw, true));
   int major = 0, minor = 0;
   doc->get_pdf_version(&major, &minor);
   std::string version_str;
@@ -178,10 +170,10 @@ List poppler_pdf_info (RawVector x, std::string opw, std::string upw) {
 // [[Rcpp::export]]
 List poppler_pdf_data (RawVector x, std::string opw, std::string upw) {
 #ifdef POPPLER_HAS_PAGE_TEXT_LIST
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   Rcpp::List out(doc->pages());
   for(int i = 0; i < doc->pages(); i++){
-    page *p(doc->create_page(i));
+    std::unique_ptr<poppler::page> p(doc->create_page(i));
     if(!p) continue; //missing page
     std::vector<text_box> boxes = p->text_list();
     CharacterVector text(boxes.size());
@@ -216,10 +208,10 @@ List poppler_pdf_data (RawVector x, std::string opw, std::string upw) {
 
 // [[Rcpp::export]]
 CharacterVector poppler_pdf_text (RawVector x, std::string opw, std::string upw) {
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   CharacterVector out(doc->pages());
   for(int i = 0; i < doc->pages(); i++){
-    page *p(doc->create_page(i));
+    std::unique_ptr<poppler::page> p(doc->create_page(i));
     if(!p) continue; //missing page
     page::text_layout_enum show_text_layout = page::physical_layout;
 
@@ -244,9 +236,41 @@ CharacterVector poppler_pdf_text (RawVector x, std::string opw, std::string upw)
   return out;
 }
 
+// [[Rcpp::export]]
+DataFrame poppler_pdf_pagesize (RawVector x, std::string opw, std::string upw) {
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
+  int len = doc->pages();
+  Rcpp::NumericVector top(len);
+  Rcpp::NumericVector right(len);
+  Rcpp::NumericVector bottom(len);
+  Rcpp::NumericVector left(len);
+  Rcpp::NumericVector width(len);
+  Rcpp::NumericVector height(len);
+
+  for(int i = 0; i < len; i++){
+    std::unique_ptr<poppler::page> p(doc->create_page(i));
+    if(!p) continue; //missing page
+    rectf rect(p->page_rect());
+    top[i] = rect.top();
+    bottom[i] = rect.bottom();
+    left[i] = rect.left();
+    right[i] = rect.right();
+    width[i] = rect.width();
+    height[i] = rect.height();
+  }
+  return DataFrame::create(
+    _["top"] = top,
+    _["right"] = right,
+    _["bottom"] = bottom,
+    _["left"] = left,
+    _["width"] = width,
+    _["height"] = height
+  );
+}
+
 // [[Rcpp::export]]
 List poppler_pdf_fonts (RawVector x, std::string opw, std::string upw) {
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   std::vector<font_info> fonts = doc->fonts();
   CharacterVector fonts_name;
   CharacterVector fonts_type;
@@ -270,7 +294,7 @@ List poppler_pdf_fonts (RawVector x, std::string opw, std::string upw) {
 
 // [[Rcpp::export]]
 List poppler_pdf_files (RawVector x, std::string opw, std::string upw) {
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   List out = List();
   if(doc->has_embedded_files()){
     std::vector<embedded_file*> files = doc->embedded_files();
@@ -294,9 +318,9 @@ List poppler_pdf_files (RawVector x, std::string opw, std::string upw) {
 
 // [[Rcpp::export]]
 List poppler_pdf_toc(RawVector x, std::string opw, std::string upw) {
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   List out = List();
-  toc *contents = doc->create_toc();
+  std::unique_ptr<poppler::toc> contents(doc->create_toc());
   if(!contents)
     return List();
   return item_to_list(contents->root());
@@ -307,14 +331,14 @@ RawVector poppler_render_page(RawVector x, int pagenum, double dpi, std::string
                               bool antialiasing = true, bool text_antialiasing = true) {
   if(!page_renderer::can_render())
     throw std::runtime_error("Rendering not supported on this platform!");
-  document *doc = read_raw_pdf(x, opw, upw);
-  page *p(doc->create_page(pagenum - 1));
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
+  std::unique_ptr<poppler::page> p(doc->create_page(pagenum - 1));
   if(!p)
     throw std::runtime_error("Invalid page.");
   page_renderer pr;
   pr.set_render_hint(page_renderer::antialiasing, antialiasing);
   pr.set_render_hint(page_renderer::text_antialiasing, text_antialiasing);
-  image img = pr.render_page(p, dpi, dpi);
+  image img = pr.render_page(p.get(), dpi, dpi);
   if(!img.is_valid())
     throw std::runtime_error("PDF rendering failure.");
   size_t len = img.bytes_per_row() * img.height();
@@ -337,19 +361,19 @@ std::vector<std::string> poppler_convert(RawVector x, std::string format, std::v
                           bool antialiasing = true, bool text_antialiasing = true, bool verbose = true) {
   if(!page_renderer::can_render())
     throw std::runtime_error("Rendering not supported on this platform!");
-  document *doc = read_raw_pdf(x, opw, upw);
+  std::unique_ptr<poppler::document> doc(read_raw_pdf(x, opw, upw));
   for(size_t i = 0; i < pages.size(); i++){
     int pagenum = pages[i];
     std::string filename = names[i];
     if(verbose)
       Rprintf("Converting page %d to %s...", pagenum, filename.c_str());
-    page *p(doc->create_page(pagenum - 1));
+    std::unique_ptr<poppler::page> p(doc->create_page(pagenum - 1));
     if(!p)
       throw std::runtime_error("Invalid page.");
     page_renderer pr;
     pr.set_render_hint(page_renderer::antialiasing, antialiasing);
     pr.set_render_hint(page_renderer::text_antialiasing, text_antialiasing);
-    image img = pr.render_page(p, dpi, dpi);
+    image img = pr.render_page(p.get(), dpi, dpi);
     if(!img.is_valid())
       throw std::runtime_error("PDF rendering failure.");
     if(!img.save(filename, format, dpi))
@@ -360,7 +384,7 @@ std::vector<std::string> poppler_convert(RawVector x, std::string format, std::v
   return names;
 }
 
-void error_callback(const std::string &msg, void *context){
+static void error_callback(const std::string &msg, void *context){
   Rcpp::Function err_cb = Rcpp::Environment::namespace_env("pdftools")["err_cb"];
   err_cb(Rcpp::String(msg));
 }
